require_relative 'player'
require_relative 'treasure_trove.rb'
module StudioGame
  describe Player do
    context do
      before do
        @player = Player.new("Larry", 150)
      end

    it "has a capitalized name" do
      expect(@player.name).to eq("Larry")
    end

    it "has a string representation" do
      @player.found_treasure(Treasure.new(:hammer, 50))
      @player.found_treasure(Treasure.new(:hammer, 50))

      expect(@player.to_s).to  eq("I am Larry with health = 150, points = 100, and score = 250.")
    end

    it "has an initial health" do
      expect(@player.health).to eq(150)
    end
  end

  context  "health restarts at zero" do
    before do
      @player = Player.new("larry",0)
      @player.found_treasure(Treasure.new(:hammer, 50))
      @player.found_treasure(Treasure.new(:hammer, 50))
    end

    it "computes a score as the sum of its health and its points" do

      @player.score.should == 100
    end

    it "increases health by 10 when w00ted" do
      @player.w00t

      expect(@player.score).to eq(110)
    end

    it "decreases health by 15 when blammed" do
      @player.blam

      expect(@player.score).to eq(85)
     end
  end

     context "when a player is strong" do
      before do
        @player = Player.new("Larry", 150)
      end

      it "player is strong" do
        expect(@player.strong?).to be true
      end
     end

     context "when a player is weak" do
      before do
        @player = Player.new("Larry", 100)
      end

      it "player is strong" do
        expect(@player.strong?).to be false
      end
     end

     context "in a collection of players" do
      before do
        @player1 = Player.new("moe", 100)
        @player2 = Player.new("larry", 200)
        @player3 = Player.new("curly", 300)

        @players = [@player1, @player2, @player3]
      end

      it "is sorted by decreasing score" do
        @players.sort.should == [@player3, @player2, @player1]
      end
    end

    context "treasure points" do
      before do
        @player = Player.new("Larry", 0)
      end

        it "computes points as the sum of all treasure points" do
        @player.points.should == 0
        #
        @player.found_treasure(Treasure.new(:hammer, 50))

        @player.points.should == 50

        @player.found_treasure(Treasure.new(:crowbar, 400))

        @player.points.should == 450

        @player.found_treasure(Treasure.new(:hammer, 50))

        @player.points.should == 500
      end
    end

    it "yields each found treasure and its total points" do
      @player = Player.new("Larry", 150)
      @player.found_treasure(Treasure.new(:skillet, 100))
      @player.found_treasure(Treasure.new(:skillet, 100))
      @player.found_treasure(Treasure.new(:hammer, 50))
      @player.found_treasure(Treasure.new(:bottle, 5))
      @player.found_treasure(Treasure.new(:bottle, 5))
      @player.found_treasure(Treasure.new(:bottle, 5))
      @player.found_treasure(Treasure.new(:bottle, 5))
      @player.found_treasure(Treasure.new(:bottle, 5))

      yielded = []
      @player.each_found_treasure do |treasure|
        yielded << treasure
      end

      yielded.should == [
        Treasure.new(:skillet, 200),
        Treasure.new(:hammer, 50),
        Treasure.new(:bottle, 25)
     ]
    end
  end
end
